package com.loafle.commons.rpc.protocol.json;

import java.lang.reflect.Type;

import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCError;
import com.loafle.commons.rpc.protocol.RPCServerRequestCodec;

/**
 * JSONRPCServerRequestCodec
 */
class JSONRPCServerRequestCodec implements RPCServerRequestCodec {
  private Gson gson;
  private JSONServerRequest request;

  public JSONRPCServerRequestCodec(Gson gson, byte[] buff) throws RPCException {
    this.gson = gson;
    try {
      this.request = this.gson.fromJson(new String(buff), JSONServerRequest.class);
    } catch (JsonSyntaxException e) {
      e.printStackTrace();
    }
  }

  public String method() {
    return this.request.method;
  }

  public Object[] params(Type[] paramTypes) throws RPCException {
    if (null == paramTypes && null == this.request.params) {
      return null;
    }

    if ((null != paramTypes && null == this.request.params) || null == paramTypes && null != this.request.params) {
      throw new RPCException();
    }

    if (paramTypes.length != this.request.params.size()) {
      throw new RPCException();
    }

    Object[] result = new Object[paramTypes.length];

    for (int i = 0; i < paramTypes.length; i++) {
      Type paramType = paramTypes[i];
      String param = this.request.params.get(i);

      result[i] = this.gson.fromJson(param, paramType);
    }

    return result;
  }

  public boolean hasResponse() {
    return null != this.request.id;
  }

  public byte[] response(Object reply, Error error) throws RPCException {
    RPCError rpcError = null;
    if (null != error) {
      rpcError = new RPCError(1, "", error);
    }
    JSONServerResponse response = new JSONServerResponse(JSONRPC.version, reply, rpcError);
    response.id = this.request.id;

    String json = this.gson.toJson(response);
    return json.getBytes();
  }
}